@@ -1,6 +1,7 @@
 /* decrypt-data.c - Decrypt an encrypted data packet
  * Copyright (C) 1998-2001, 2005-2006, 2009 Free Software Foundation, Inc.
  * Copyright (C) 1998-2001, 2005-2006, 2009, 2018 Werner Koch
+ * Copyright (C) 2020 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -43,7 +44,7 @@ static int decode_filter ( void *opaque, int control, IOBUF a,
 /* Our context object.  */
 struct decode_filter_context_s
 {
-  /* Redcounter (max value is 2).  We need it because we do not know
+  /* Recounter (max value is 2).  We need it because we do not know
    * whether the iobuf or the outer control code frees this object
    * first.  */
   int  refcount;
@@ -234,8 +235,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   byte *p;
   int rc=0, c, i;
   byte temp[32];
-  unsigned int blocksize;
-  unsigned int nprefix;
+  unsigned blocksize;
+  unsigned nprefix;
 
   *compliance_error = 0;
 
@@ -281,11 +282,11 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
     }
 
   write_status_printf (STATUS_DECRYPTION_INFO, "%d %d %d",
-                       ed->mdc_method, dek->algo, ed->aead_algo);
+                       ed->mdc_method, dek->algo, 0);
 
   if (opt.show_session_key)
     {
-      char numbuf[30];
+      char numbuf[25];
       char *hexbuf;
 
       if (ed->aead_algo)
@@ -428,11 +429,11 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
         {
           log_info (_("WARNING: message was encrypted with"
                       " a weak key in the symmetric cipher.\n"));
-          rc=0;
+          rc = 0;
         }
       else if (rc)
         {
-          log_error ("key setup failed: %s\n", gpg_strerror (rc) );
+          log_error("key setup failed: %s\n", gpg_strerror (rc) );
           goto leave;
         }
 
@@ -498,7 +499,7 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
       rc = get_output_file ("", 0, ed->buf, &filename, &fp);
       if (! rc)
         {
-          iobuf_t output = iobuf_esopen (fp, "w", 0, 0);
+          iobuf_t output = iobuf_esopen (fp, "w", 0);
           armor_filter_context_t *afx = NULL;
 
 	  es_setbuf (fp, NULL);
@@ -560,8 +561,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
           || datalen != 20
           || memcmp (gcry_md_read (dfx->mdc_hash, 0), dfx->holdback+2, datalen))
         rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
-      /* log_printhex("MDC message:", dfx->holdback, 22); */
-      /* log_printhex("MDC calc:", gcry_md_read (dfx->mdc_hash,0), datalen); */
+      /* log_printhex(dfx->holdback, 22, "MDC message:"); */
+      /* log_printhex(gcry_md_read (dfx->mdc_hash,0), datalen, "MDC calc:"); */
     }
 
  leave:
@@ -685,10 +686,8 @@ aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
   /* Decrypt the buffer.  This first requires a loop to handle the
    * case when a chunk ends within the buffer.  */
   if (DBG_FILTER)
-    log_debug ("decrypt: chunklen=%llu total=%llu size=%zu len=%zu%s\n",
-               (unsigned long long)dfx->chunklen,
-               (unsigned long long)dfx->total,
-               size, len,
+    log_debug ("decrypt: chunklen=%ju total=%ju size=%zu len=%zu%s\n",
+               dfx->chunklen, dfx->total, size, len,
                dfx->eof_seen? " eof":"");
 
   while (len && dfx->chunklen + len >= dfx->chunksize)
@@ -724,8 +723,8 @@ aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
       len -= n;
 
       if (DBG_FILTER)
-        log_debug ("ndecrypted: %zu (nchunk=%llu) bytes left: %zu at off=%zu\n",
-                   totallen, (unsigned long long)dfx->chunklen, len, off);
+        log_debug ("ndecrypted: %zu (nchunk=%ju) bytes left: %zu at off=%zu\n",
+                   totallen, dfx->chunklen, len, off);
 
       /* Check the tag.  */
       if (len < 16)
@@ -806,8 +805,7 @@ aead_underflow (decode_filter_ctx_t dfx, iobuf_t a, byte *buf, size_t *ret_len)
       dfx->chunklen += len;
       dfx->total += len;
       if (DBG_FILTER)
-        log_debug ("ndecrypted: %zu (nchunk=%llu)\n",
-                   totallen, (unsigned long long)dfx->chunklen);
+        log_debug ("ndecrypted: %zu (nchunk=%ju)\n", totallen, dfx->chunklen);
     }
 
   if (dfx->eof_seen)
@@ -938,7 +936,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
       n = fill_buffer (dfx, a, buf, 44, 22);
       if (n == 44)
         {
-          /* We have enough stuff - flush the holdback buffer.  */
+          /* We have enough stuff - flush the deferred stuff.  */
           if ( !dfx->holdbacklen )  /* First time. */
             {
               memcpy (buf, buf+22, 22);
@@ -948,7 +946,6 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
             {
               memcpy (buf, dfx->holdback, 22);
 	    }
-
           /* Fill up the buffer. */
           n = fill_buffer (dfx, a, buf, size, n);
 
@@ -958,7 +955,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
           memcpy (dfx->holdback, buf+n, 22 );
           dfx->holdbacklen = 22;
 	}
-      else if ( !dfx->holdbacklen ) /* EOF seen but empty holdback. */
+      else if ( !dfx->holdbacklen )  /* EOF seen but empty holdback buffer. */
         {
           /* This is bad because it means an incomplete hash. */
           n -= 22;
